#!/usr/bin/env python
import paths as pt
from tools import file_reader, file_writer, feature_maker
from tools import preprocessor
import pandas as pd
import yaml
from pathlib import Path

def main(ats_resolution: int = None):
    clusters = file_reader.read_csv(pt.INTERIM_DATA_DIR, 'cl.csv',
                              converters={'CitizenId': str, 'Cluster': int})
    screenings = file_reader.read_csv(pt.INTERIM_DATA_DIR, 'screenings.csv',
                              converters={'CitizenId': str})
    fall_data = pd.DataFrame(file_reader.read_pickle(pt.INTERIM_DATA_DIR, 'fd.pkl'))
    fall_data = fall_data.drop_duplicates(["CitizenId", "Date"])
    
    with open(Path.joinpath(pt.CONFIGS_DIR, "settings.yaml"), 'r') as stream:
        settings = yaml.safe_load(stream)
    if ats_resolution == None:
        ats_resolution = settings['ats_resolution']
    ex_resolution = settings['ex_resolution']
    standard_fts = settings['standard_features']
    extended_fts = settings['extended_features']
        
    for target_name in ['Complete', 'Compliance', 'Fall', 'Risk']:
        df = screenings.copy()
        df['Cluster'] = clusters['Cluster']
        
        # Split cat columns by ATS resolution
        df = preprocessor.split_cat_columns(df, col_to_split='Ats', tag='Ats',
                                            resolution=ats_resolution)
        if target_name == "Risk":
            df = preprocessor.split_cat_columns(df, col_to_split='Ex', tag='Ex',
                                                resolution=ex_resolution)
        
        # Encode target label
        if target_name == 'Complete':
            df = feature_maker.make_complete_feature(df, settings)
        elif target_name == 'Compliance':
            df = feature_maker.make_compliance_feature(df, settings)
        elif target_name == 'Fall':
            df = feature_maker.make_fall_feature(df, settings)
        else:
            df = feature_maker.make_risk_feature(df, fall_data, settings)
        
        # One-hot-encode gender variable
        object_cols = ['Gender']
        df_enc = preprocessor.one_hot_encode(df, object_cols)
        df = pd.concat([df.drop(object_cols, axis=1), df_enc], axis=1)
    
        # Concat dataframe in proper order
        if target_name in ["Complete", "Compliance", "Fall"]:
            ats_cols = df.filter(regex='Ats', axis=1)
            df = pd.concat([df[standard_fts], ats_cols, df[[target_name]]], axis=1)
        else:
            ats_ex_cols = df.filter(regex='Ats|Ex', axis=1)
            df = pd.concat([df[extended_fts], ats_ex_cols, df[[target_name]]], axis=1)

        if settings['use_real_ats_names']:
            if target_name in ["Complete", "Compliance", "Fall"]:
                ats = file_reader.read_csv(pt.REFERENCES_DIR, 'ats.csv',
                                        converters={'ats_id': str})
                df = preprocessor.replace_cat_values(df, ats)
            else:
                ats = file_reader.read_csv(pt.REFERENCES_DIR, 'ats.csv',
                                           converters={'ats_id': str})
                ex = file_reader.read_csv(pt.REFERENCES_DIR, 'ex.csv',
                                          converters={'ex_id': str})
                df = preprocessor.replace_cat_values(df, ats)
                df = preprocessor.replace_cat_values(df, ex)
                
        file_writer.write_csv(df, pt.PROCESSED_DATA_DIR, f'{target_name.lower()}.csv')
        
if __name__ == "__main__":
    main()