import pytest
import sklearn.utils as skut
import pandas as pd
import paths as pt
from src.tools import preprocessor, feature_maker, classifiers, file_reader
from src.tools.classifiers import RfClassifier, ClassifierResult

@pytest.fixture(scope="module")
def get_data():
    return file_reader.read_csv(pt.TESTS_FILES_DIR, 'screenings.csv')

def test_predict_complete(get_data):
    settings = dict({'threshold_weeks': 8, 'threshold_training': 10})
    
    df = feature_maker.make_complete_feature(get_data, settings)
    df = preprocessor.split_cat_columns(df, col_to_split='Ats', tag='Ats', resolution=10)

    ats_cols = df.filter(regex='Ats', axis=1)
    general_cols = df[['Gender', 'BirthYear', 'LoanPeriod']]
    df = pd.concat([general_cols, ats_cols, df[['Complete']]], axis=1)
    df = df.dropna()
    
    y = df["Complete"]
    X = df.drop(['Complete'], axis=1)
    
    assert_split(X, y)
    
    result = RfClassifier(X, y).evaluate()
    assert_result(result)
    
def test_predict_compliance(get_data):
    settings = dict({'threshold_weeks': 8, 'threshold_training': 10})
    
    df = feature_maker.make_compliance_feature(get_data, settings)
    df = preprocessor.split_cat_columns(df, col_to_split='Ats', tag='Ats', resolution=10)
    
    ats_cols = df.filter(regex='Ats', axis=1)
    general_cols = df[['Gender', 'BirthYear', 'LoanPeriod']]
    df = pd.concat([general_cols, ats_cols, df[['Compliance']]], axis=1)
    df = df.dropna()
    
    y = df["Compliance"]
    X = df.drop(['Compliance'], axis=1)
    
    assert_split(X, y)
    
    result = RfClassifier(X, y).evaluate()
    assert_result(result)
    
def test_predict_fall(get_data):
    settings = dict({'threshold_weeks': 8, 'threshold_training': 10})
    
    df = feature_maker.make_fall_feature(get_data, settings)
    df = preprocessor.split_cat_columns(df, col_to_split='Ats', tag='Ats', resolution=10)

    ats_cols = df.filter(regex='Ats', axis=1)
    general_cols = df[['Gender', 'BirthYear', 'LoanPeriod']]
    df = pd.concat([general_cols, ats_cols, df[['Fall']]], axis=1)
    df = df.dropna()
    
    y = df["Fall"]
    X = df.drop(['Fall'], axis=1)
    
    assert_split(X, y)
    
    result = RfClassifier(X, y).evaluate()
    assert_result(result)
    
def assert_result(result: ClassifierResult):
    assert result is not None    

def assert_split(X, y):
    skut.check_X_y(X, y.values.ravel())
    skut.check_consistent_length(X, y.values.ravel())